/*
 * Copyright (c) 2008-2011 CNRS/LAAS
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <portLib.h>
#include <h2mathLib.h>

#include "server/locoDemoHeader.h"

/* Specific includes */
#include "locoDemoConst.h"

#include "locoDemoPom.h"

static POSTER_ID pomPosterId = NULL;

void 
initPomPosters(void)
{
	POM_ME_POS *mePos = &SDI_F->mePos;

	mePos->kind = POM_ME_ABSOLUTE;
	mePos->date1 = -1;
	mePos->confidence = 0.2;	/* XXX used? */
	mePos->main.var.voo     = LOCODEMO_VAR_TT;
	mePos->main.var.vpp     = LOCODEMO_VAR_TT;
	mePos->main.var.vrr     = LOCODEMO_VAR_TT;
	mePos->main.var.vxx     = LOCODEMO_VAR_XX;
	mePos->main.var.vyy     = LOCODEMO_VAR_YY;
	mePos->main.var.vzz     = LOCODEMO_VAR_XX;
}



/*----------------------------------------------------------------------
 * SetPom
 *
 * Description:
 *
 * Reports:	OK
 *		S_jloco_POSTER_NOT_FOUND
 */

/* setPom - codel EXEC of SetPom
   Returns: EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
setPom(GENPOS_POSTER_NAME * posterName, int * report)
{
	POSTER_ID id;

	if (posterFind(posterName->name, &id) != OK) {
		logMsg("posterFind(%s) failed\n", posterName->name);
		*report = S_locoDemo_POSTER_NOT_FOUND;
	} else
		pomPosterId = id;
	return ETHER;
}

/*----------------------------------------------------------------------
 * updatePomPosters
 *
 * Description:
 *  Updates data written to the POM poster
 */
void
updatePomPosters(void)
{
	POM_ME_POS *mePos = &SDI_F->mePos;
	GENPOS_CART_STATE *configId;
	POM_POS pomPos;

	if (pomPosterId != NULL) {
		if(locoDemoPOM_POSPosterRead(pomPosterId, &pomPos) == OK) {
			mePos->date1 = pomPos.date;
		} else {
			logMsg("locoDemoPOM_POSPosterRead() failed\n");
			pomPosterId = NULL;
			mePos->date1 = -1;
		}
	} else
		mePos->date1 = -1;
	
	configId = &locoDemoDataStrId->robot;
	mePos->main.euler.yaw = configId->theta;
	mePos->main.euler.x   = configId->xRob;
	mePos->main.euler.y   = configId->yRob;
	
	/* speeds are mostly zero, just update the linear 
	   and angular components */
	mePos->VLocal.euler.x   = configId->v;
	mePos->VLocal.euler.yaw = configId->w;

	/* all variances and co-variances remain zero */
}
