/*
 * Copyright (c) 2008-2011 CNRS/LAAS
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
/**
 ** locoDemoTrackTaskCodels.c
 **
 ** Codels called by execution task locoDemoTrackTask
 **
 ** Author: 
 ** Date: 
 **
 **/

#include <math.h>
#include <portLib.h>
#include <h2mathLib.h>

#include "server/locoDemoHeader.h"

#include "locoDemoConst.h"

/*------------------------------------------------------------------------
 * Track
 *
 * Description: 
 *
 * Reports:      OK
 *              S_locoDemo_POSTER_NOT_FOUND
 *              S_locoDemo_BAD_REF
 *              S_locoDemo_CMD_STOP_TRACK
 */

/* findTrackedPoster  -  codel START of Track
   Returns:  START EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
findTrackedPoster(GENPOS_POSTER_NAME *trackedPoster, int *report)
{
	POSTER_ID posterTrackedId;
	
	/* Identificateur du poster */
	if (posterFind((char *) trackedPoster, &posterTrackedId) != OK)  {
		*report = S_locoDemo_POSTER_NOT_FOUND;
		return(END); 
	}
	locoDemoDataStrId->trackStr.posterId = (int) posterTrackedId;
	locoDemoDataStrId->mode = LOCODEMO_TRACK;
	return(EXEC);
}

/* pumpReference  -  codel EXEC of Track
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
pumpReference(GENPOS_POSTER_NAME *trackedPoster, int *report)
{
	GENPOS_CART_REF locoDemoOrders;
	GENPOS_CART_REF *refId = &locoDemoDataStrId->ref;
	GENPOS_CART_STATE *configId = &locoDemoDataStrId->robot;
	
	/* Interruption interne */
	if (locoDemoDataStrId->trackStr.trackMode != GENPOS_TRACK_CONFIG) {
		*report = EXEC_TASK_BILAN(LOCODEMO_MOTIONTASK_NUM);
		return(END);
	}
	
	/* Lire les consignes sur le poster externe */
	if (posterRead ((POSTER_ID)locoDemoDataStrId->trackStr.posterId, 0, 
		(void *) &locoDemoOrders,
		sizeof (GENPOS_CART_REF)) == sizeof (GENPOS_CART_REF)) {
		
		/* Verifier la validite des arguments */
		if (locoDemoOrders.vmax > LOCODEMO_VMAX)
			locoDemoOrders.vmax = LOCODEMO_VMAX;
		if (locoDemoOrders.wmax > LOCODEMO_WMAX)
			locoDemoOrders.wmax = LOCODEMO_VMAX;
		if (fabs(locoDemoOrders.v) > locoDemoOrders.vmax)
			locoDemoOrders.v = locoDemoOrders.v > 0 ?
			    locoDemoOrders.vmax : -locoDemoOrders.vmax;
		if (fabs(locoDemoOrders.w) > locoDemoOrders.wmax)
			locoDemoOrders.w = locoDemoOrders.w > 0 ?
			    locoDemoOrders.wmax : -locoDemoOrders.wmax;

		refId->backFlag = (locoDemoOrders.v > 0 ?
		    GENPOS_FORWARD_MOTION : GENPOS_BACKWARD_MOTION);
		refId->v = fabs(locoDemoOrders.v);
		refId->vt = 0;
		refId->w = locoDemoOrders.w;
		refId->vmax = locoDemoOrders.vmax;
		refId->wmax = locoDemoOrders.wmax;
		refId->linAccelMax = locoDemoOrders.linAccelMax;
		refId->angAccelMax = locoDemoOrders.angAccelMax;
		refId->x = locoDemoOrders.x;
		refId->y = locoDemoOrders.y;
		refId->theta =  angleLimit(locoDemoOrders.theta);
		refId->dataType = GENPOS_POS_AND_SPEED_DATA;
		return(EXEC);
	}
	*report = errnoGet();
	return(END);
}

/* smoothStopTrack  -  codel END of Track
   Returns:  END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
smoothStopTrack(GENPOS_POSTER_NAME *trackedPoster, int *report)
{
	GENPOS_CART_REF *refId;	       /* Id structure de consignes */
	GENPOS_CART_STATE *robotId;	/* Id config du robot */
	double dtArret;                /* Temps d'arret */
	double dtRetard;               /* Temps de prise en compte de la consigne */
	double dsArret;                /* Longueur d'arret */
	double dtLoco;                 /* Periode d'asserv de la loco */
	double angle;                  /* Moyenne entre angles d'init et d'arret */
	double theta;
	
	/* Obtenir les pointeurs vers les structures de donnees de la locomotion */
	robotId = &(locoDemoDataStrId->robot);
	refId = &locoDemoDataStrId->ref;
	
	/* Determination de la periode d'asserv de la loco */
	dtLoco = EXEC_TASK_PERIOD(LOCODEMO_TRACKTASK_NUM);
	
	/* Temps d'arret minimal (tps de decel) */
	dtArret = MAX (fabs(robotId->v) / LOCODEMO_ACCEL_LIN_MAX,
	    fabs(robotId->w) / LOCODEMO_ACCEL_ANG_MAX);
	
	/* Temps de prise en compte (v = cst) */
	dtRetard =  1 * dtLoco;
	
	/* Angle final (avec periode de prise en compte) */
	theta = robotId->theta + robotId->w * (dtArret / 2 + dtRetard);
	
	/* Angle moyen */
	angle = (robotId->theta + theta) / 2;
	
	/* Longueur d'arret */
	dsArret = robotId->v * (dtArret / 2 + dtRetard);
	
	/* Prevision position du centre */
	refId->x = robotId->xRob + dsArret * cos (angle);
	refId->y = robotId->yRob + dsArret * sin (angle);
	
	/* Vitesses de consigne nulles */
	refId->v = 0. ;
	refId->w = 0. ;
	
	/* Indiquer l'arret du tracking */
	refId->dataType = GENPOS_POS_AND_SPEED_DATA;
	locoDemoDataStrId->trackStr.trackMode = GENPOS_NO_TRACKING;
	locoDemoDataStrId->mode = LOCODEMO_IDLE;
	return (ETHER);	
}
