/*
 * Copyright (c) 2008-2011 CNRS/LAAS
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
#ifndef  LOCODEMO_CONST_H
#define  LOCODEMO_CONST_H


/*---------- PARAMETRES PAR DEFAUT DE LA BOUCLE D'ASSERVISSEMENT ------------*/

/* Vitesse lineaire max du robot */
#define  LOCODEMO_VMAX                 0.9

/* Vitesse angulaire max du robot */
#define  LOCODEMO_WMAX                   1.

/* Acceleration max lineaire du robot */
#define  LOCODEMO_ACCEL_LIN_MAX                1.

/* Acceleration max angulaire du robot */
#define  LOCODEMO_ACCEL_ANG_MAX               (3. * M_PI )

/* Correction vitesse lineaire, gain proportionnel erreur longitudinale */
#define  LOCODEMO_KP_LONGIT                    1.0
#define  LOCODEMO_KI_LONGIT                    1	//OK=1
#define  LOCODEMO_KD_LONGIT                    0	//OK=0

/* Correction vitesse angulaire, gain proportionnel, erreur transversale */
#define    LOCODEMO_KP_TRANSV                    1.0
#define  LOCODEMO_KI_TRANSV                    6 	//OK=6
#define  LOCODEMO_KD_TRANSV                    0	// OK=0

/*Correction vitesse angulaire, gain proportionnel & integral, erreur angulaire 
   (si on utilise contLawCartConfigControl() pour l'asservicement) ???? */
#define 	LOCODEMO_KP_ANG			2 //OK=2
#define 	LOCODEMO_KI_ANG			2 //OK=2

/* Erreurs max acceptés (m & rad)*/

#define 	LOCODEMO_LONGIT_MAX_GAP 		0.2
#define 	LOCODEMO_TRANSV_MAX_GAP		0.2
#define 	LOCODEMO_ANG_MAX_GAP		0.2

/*erreurs max raportées (pour l'asservissement proportionnel) */

#define 	LOCODEMO_LONGIT_ERROR_MAX		0.5	//if dl > longitErrorMax then SdL = 0
#define 	LOCODEMO_TRANSV_ERROR_MAX		0.5	//if dl > longitErrorMax then SdL = 0
#define 	LOCODEMO_ANG_ERROR_MAX		0.5	//if dl > longitErrorMax then SdL = 0

/*Somme des erreurs max raportées (pour l'asservissement integral)*/

#define 	LOCODEMO_LONGIT_MAX_ERR_SUM	1.0	//uper bound for SdL
#define 	LOCODEMO_TRANSV_MAX_ERR_SUM	1.0	//uper bound for SdT
#define 	LOCODEMO_ANG_MAX_ERR_SUM		1.0	//uper bound for SdA

#define LOCODEMO_CNTR_POINT_DIST 0.0

/*---------- Used only by pilo -------------------------------------------*/

#define LOCODEMO_MOT_AXIS_WIDTH  0.2

/* Default uncertainties for pom */
#define LOCODEMO_VAR_XX     0.001
#define LOCODEMO_VAR_YY     0.001
#define LOCODEMO_VAR_TT     0.0001
#define LOCODEMO_VAR_QCOEFF 0.05

#endif

